recursive subroutine bspl2(x,k,t,r,y,m,n)
!B-spline basis functions and their derivatives
!INPUT:
!   x	(m)	 Input grid.
!   k   (integer)  Spline order.
!   t   (n)	 Knots, strictly increasing sequence that INCLUDES interval endpoints.
!   r	(integer) Order of derivative.
!
!OUTPUT:
!   y   (m x n+k-2) Basis function (or derivative) values at X
!
! Version: May 2010 in Matlab, December 2011 in Fortran

implicit none
! declare arguments
integer, intent(in) :: k, r, m, n
double precision, intent(in) :: x(m), t(n)
double precision, intent(out) :: y(m,n+k-2)
! declare locals
integer :: l, i, j, o, nn
integer, allocatable :: idx(:)
double precision :: saved, term 
double precision, allocatable :: tt(:), b(:), dr(:), dl(:), yrec(:,:), msp(:,:)

print*, r
y = 0.d0

if (r==0) then

    allocate(tt(n+2*(k-1)),b(k),dr(k-1),dl(k-1))
    tt(1:k-1) = t(1)
    tt(k:k+n-1) = t
    tt(k+n:k+n+k-2) = t(n)
    nn = size(tt,1)
    allocate(idx(nn))
    do l = 1,m
        b(1) = 1
		forall(i=1:nn) idx(i) = i
		where(tt>x(l)) idx = 0
		i = maxval(idx) 		! i = find(tt<=x(l),1,'last') in Matlab
        if (i==nn) i = nn-k
        do j = 1,k-1
            dr(j) = tt(i+j)-x(l)
            dl(j) = x(l)-tt(i+1-j)
            saved = 0.d0
            do o = 1,j
                term = b(o)/(dr(o)+dl(j+1-o))
                b(o) = saved + dr(o)*term
                saved = dl(j+1-o)*term
            end do
            b(j+1) = saved
        end do
        y(l,i-k+1:i) = b
    end do
    
else

    allocate(tt(n+2*(k-2)),yrec(m,n+k-3),msp(m,n+k-3))
    tt(1:k-2) = t(1)
    tt(k-1:k-1+n-1) = t
    tt(k-1+n:k-1+n+k-3) = t(n)
    call bspl2(x,k-1,t,r-1,yrec,m,n)
    forall(i=1:m) msp(i,:) = (real(k-1)/(tt(k:n+2*(k-2))-tt(1:n+k-3)))*yrec(i,:)
    y(:,1) =  - msp(:,1)
    y(:,2:n+k-3) = msp(:,1:n+k-4) - msp(:,2:n+k-3)
    y(:,n+k-2) =  msp(:,n+k-3)
    
end if

end subroutine bspl2
