subroutine compsums(s,t,x,y,rngs,rngt,nks,nkt,ors,ort,Wmc0,Emc0,mx,my,Cx,Cy,s2x,s2y,A,Sw,sqSw,invSw,Se,sqSe, &
					taux0,tauy0,n,m1,m2,Nmc,dx,dy,px,py,qx,qy,rx,ry, &	! the rest are output arguments
					Sw0w0,Snormw2,Sr1_2,Sz0w0,SBBx,SBRx,Snormx2,SuuBBx,SuBxr6, &
				    SBBy,SBRy,Snormy2,SvvBBy,SvByr7,f,pred_w,pred_z,errflag)
! Computes sums of expectations (for equal time grids)
! NOTE the difference between vectors and 1-dim row or column matrices below
!	s	(m1)	Time grid of the x's
!	t	(m2)	Time grid of the y's
!	x	(n x m1)		X data
!	y	(n x m2)	Y data
!	rngs	(2)		Time range of interest for the x's (lower and upper bounds)
!	rngt	(2)		Time range of interest for the y's (lower and upper bounds)
!	nks	(scalar)	Number of B-spline knots for the x's
!	nkt	(scalar)	Number of B-spline knots for the y's
!	ors	(scalar)	Spline order for the x's
!	ort	(scalar)	Spline order for the y's
!	Wmc0	(dx x Nmc)	Basic MC sample for the W's
!	Emc0	(dy	x Nmc)	Basic MC sample for the E's
!	mx		(qx)	Basis coefficients for the mean of the x's
!	my		(qy)	Basis coefficients for the mean of the y's
!	Cx		(qx x px)	K-L basis coefficients for the x's
!	Cy		(qy x py)	K-L basis coefficients for the y's
!	s2x	(scalar)	Random error variance for the x's
!	s2y	(scalar)	Random error variance for the y's
!	A	(dy x dx)	Regression matrix
!	Sw	(dx x dx)	Covariance matrix of the w's
!	sqSw	(dx x dx)	Matrix square root of Sw
!	invSw	(dx x dx)	Matrix inverse of Sw
!	Se	(dy x dy)	Covariance matrix of the e's
!	sqSe	(dx x dx)	Matrix square root of Se
!	taux0	(1 x rx)	Mean warping knots for the x's
!	tauy0	(1 x ry)	Mean warping knots for the y's

! external subroutines used: jupp2, bspl2, hwarpinv2

!declare arguments
implicit none
double precision, intent(in) :: s(m1), t(m2), x(n,m1), y(n,m2), rngs(2), rngt(2), Wmc0(dx,Nmc), Emc0(dy,Nmc), &
	mx(qx), my(qy), Cx(qx,px), Cy(qy,py), s2x, s2y, A(dy,dx), Sw(dx,dx), sqSw(dx,dx), invSw(dx,dx), &
	Se(dx,dx), sqSe(dy,dy), taux0(1,rx), tauy0(1,ry)
integer, intent(in) :: nks, nkt, ors, ort
integer, intent(in) :: n, m1, m2, Nmc, dx, dy, px, py, qx, qy, rx, ry
double precision, intent(out) :: f(n), pred_w(n,dx), pred_z(n,dy)
double precision, intent(out) :: Sw0w0(dx,dx), Snormw2, Sr1_2(dy), Sz0w0(dy,dx), SBBx(qx,qx), &
	SBRx(qx), Snormx2, SuuBBx(qx,qx,px), SuBxr6(qx,px), SBBy(qy,qy), SBRy(qy), Snormy2, &
	SvvBBy(qy,qy,py), SvByr7(qy,py)
!declare locals
double precision :: muthx(1,rx), muthy(1,ry), muw(dx), muz(dy), sknots(nks+2), tknots(nkt+2)
double precision :: Wmc(dx), Emc(dy), Zmc(dy), tauxmc(1,rx), tauymc(1,ry), fxcond, fycond, fcond, Nd, &
	tWmc(1,rx), tZmc(1,ry), vtauxmc(rx), vtauymc(ry), vtaux0(rx), vtauy0(ry)
double precision :: w0w0(dx,dx,n), normw2(n), r1_2(dy,n), z0w0(dy,dx,n), BBx(qx,qx,n), &
	BRx(qx,n), normx2(n), uuBBx(qx,qx,px,n), uBxr6(qx,px,n), BBy(qy,qy,n), &
	BRy(qy,n), normy2(n), vvBBy(qy,qy,py,n), vByr7(qy,py,n), WWt(dx,dx), ZWt(dy,dx), &
	mux6(qx), muy7(qy)
integer :: errflag, i, j, k, nux, nuy, ii, jj
double precision, allocatable :: ss(:), xx(:,:), tt(:), yy(:,:), si_star(:), ti_star(:), &
	Bx(:,:),By(:,:), mux(:), muy(:)
double precision, parameter :: pi = 3.141592654d0

! Begin
errflag = 0
 call jupp2(taux0,rngs(1),rngs(2),"direct",muthx,errflag,1,rx)
if (errflag==1) return
 call jupp2(tauy0,rngt(1),rngt(2),"direct",muthy,errflag,1,ry)
if (errflag==1) return 
muw(1:px) = 0.d0
muw(px+1:dx) = muthx(1,:)
muz(1:py) = 0.d0
muz(py+1:dy) = muthy(1,:)
forall (i=1:nks+2) sknots(i) = rngs(1) + (rngs(2)-rngs(1))*dble(i-1)/dble(nks+1)
forall (i=1:nkt+2) tknots(i) = rngt(1) + (rngt(2)-rngt(1))*dble(i-1)/dble(nkt+1)
nux = count((s>=rngs(1)).and.(s<=rngs(2)))
nuy = count((t>=rngt(1)).and.(t<=rngt(2)))
allocate(ss(nux),xx(n,nux),tt(nuy),yy(n,nuy),si_star(nux),ti_star(nuy), &
	Bx(nux,qx),By(nuy,qy),mux(nux),muy(nuy))
j = 0
do i = 1,m1
	if((s(i)>=rngs(1)).and.(s(i)<=rngs(2))) then
		j = j+1
		ss(j) = s(i)
		xx(:,j) = x(:,i)
	end if
end do
k = 0
do i = 1,m2
	if((t(i)>=rngt(1)).and.(t(i)<=rngt(2))) then
		k = k+1
		tt(k) = t(i)
		yy(:,k) = y(:,i)
	end if
end do
! Expectations
Nd = dble(Nmc)
do j = 1,Nmc
    Wmc = muw + matmul(sqSw,Wmc0(:,j))
    Emc = matmul(sqSe,Emc0(:,j))
    Zmc = muz + matmul(A,Wmc-muw) + Emc
    tWmc(1,:) = Wmc(px+1:dx)
    tZmc(1,:) = Zmc(py+1:dy)
    call jupp2(tWmc,rngs(1),rngs(2),"inverse",tauxmc,errflag,1,rx)
    if (errflag==1) return
    call jupp2(tZmc,rngt(1),rngt(2),"inverse",tauymc,errflag,1,ry)
    if (errflag==1) return
    vtauxmc = tauxmc(1,:)
    vtauymc = tauymc(1,:)
    vtaux0 = taux0(1,:)
    vtauy0 = tauy0(1,:)
    call hwarpinv2(ss,vtauxmc,vtaux0,rngs,si_star,errflag,nux,rx)
    if (errflag==1) return
    call hwarpinv2(tt,vtauymc,vtauy0,rngt,ti_star,errflag,nuy,ry)
    if (errflag==1) return
    call bspl2(si_star,ors,sknots,0,Bx,nux,nks+2)
    call bspl2(ti_star,ort,tknots,0,By,nuy,nkt+2)
    mux = matmul(Bx,mx+matmul(Cx,Wmc(1:px)))
    muy = matmul(By,my+matmul(Cy,Zmc(1:py)))
    do i = 1,n
        fxcond = dexp(-0.5d0*sum((xx(i,:)-mux)**2)/s2x)/dsqrt(2.d0*pi*s2x)**nux
        fycond = dexp(-0.5d0*sum((yy(i,:)-muy)**2)/s2y)/dsqrt(2.d0*pi*s2y)**nuy
        fcond = fxcond*fycond
        f(i) = f(i) + fcond/Nd
        forall(ii=1:dx,jj=1:dx) WWt(ii,jj) = (Wmc(ii)-muw(ii))*(Wmc(jj)-muw(jj))	! This is just W0*W0'
        w0w0(:,:,i) = w0w0(:,:,i) + WWt*fcond/Nd
        normw2(i) = normw2(i) + dot_product(Wmc-muw,matmul(invSw,Wmc-muw))*fcond/Nd
        r1_2(:,i) = r1_2(:,i) + (Zmc-muz-matmul(A,Wmc-muw))**2*fcond/Nd
        forall(ii=1:dy,jj=1:dx) ZWt(ii,jj) = (Zmc(ii)-muZ(ii))*(Wmc(jj)-muw(jj))	! This is just Z0*W0'
        z0w0(:,:,i) = z0w0(:,:,i) + ZWt*fcond/Nd
        BBx(:,:,i) = BBx(:,:,i) + matmul(transpose(Bx),Bx)*fcond/Nd
        BRx(:,i) = BRx(:,i) + matmul(transpose(Bx),xx(i,:)-matmul(Bx,matmul(Cx,Wmc(1:px))))*fcond/Nd
        BBy(:,:,i) = BBy(:,:,i) + matmul(transpose(By),By)*fcond/Nd
        BRy(:,i) = BRy(:,i) + matmul(transpose(By),yy(i,:)-matmul(By,matmul(Cy,Zmc(1:py))))*fcond/Nd
        normx2(i) = normx2(i) + sum((xx(i,:)-mux)**2)*fcond/Nd
        normy2(i) = normy2(i) + sum((yy(i,:)-muy)**2)*fcond/Nd
        mux6 = 0.d0
        do jj = 1,px
        	uuBBx(:,:,jj,i) = uuBBx(:,:,jj,i) + (Wmc(jj)**2)*matmul(transpose(Bx),Bx)*fcond/Nd
        	mux6 = mx + matmul(Cx,Wmc(1:px)) - Cx(:,jj)*Wmc(jj)
        	uBxr6(:,jj,i) = uBxr6(:,jj,i) + Wmc(jj)*matmul(transpose(Bx),xx(i,:)-matmul(Bx,mux6))*fcond/Nd
       	end do
       	muy7 = 0.d0
        do jj = 1,py
        	vvBBy(:,:,jj,i) = vvBBy(:,:,jj,i) + (Zmc(jj)**2)*matmul(transpose(By),By)*fcond/Nd
        	muy7 = my + matmul(Cy,Zmc(1:py)) - Cy(:,jj)*Zmc(jj)
			vByr7(:,jj,i) = vByr7(:,jj,i) + Zmc(jj)*matmul(transpose(By),yy(i,:)-matmul(By,muy7))*fcond/Nd
		end do
        pred_w(i,:) = pred_w(i,:) + Wmc*fcond/Nd
        pred_z(i,:) = pred_z(i,:) + Zmc*fcond/Nd
    end do
end do
! Sums
Sw0w0 = 0.d0
Snormw2 = 0.d0
Sr1_2 = 0.d0
Sz0w0 = 0.d0
SBBx = 0.d0
SBRx = 0.d0
Snormx2 = 0.d0
SuuBBx = 0.d0
SuBxr6 = 0.d0
SBBy = 0.d0
SBRy = 0.d0
Snormy2 = 0.d0
SvvBBy = 0.d0
SvByr7 = 0.d0
do i = 1,n
	if (f(i)>0.d0) then
        Sw0w0 = Sw0w0 + w0w0(:,:,i)/f(i)
        Snormw2 = Snormw2 + normw2(i)/f(i)
        Sr1_2 = Sr1_2 + r1_2(:,i)/f(i)
        Sz0w0 = Sz0w0 + z0w0(:,:,i)/f(i)
        SBBx = SBBx + BBx(:,:,i)/f(i)
        SBRx = SBRx + BRx(:,i)/f(i)
        Snormx2 = Snormx2 + normx2(i)/f(i)
        SuuBBx = SuuBBx + uuBBx(:,:,:,i)/f(i)
        SuBxr6 = SuBxr6 + uBxr6(:,:,i)/f(i)
        SBBy = SBBy + BBy(:,:,i)/f(i)
        SBRy = SBRy + BRy(:,i)/f(i)
        Snormy2 = Snormy2 + normy2(i)/f(i)
        SvvBBy = SvvBBy + vvBBy(:,:,:,i)/f(i)
        SvByr7 = SvByr7 + vByr7(:,:,i)/f(i)
        pred_w(i,:) = pred_w(i,:)/f(i)
        pred_z(i,:) = pred_z(i,:)/f(i)
	else
		f(i) = epsilon(1.d0)        
	end if
end do

deallocate(ss,xx,tt,yy,si_star,ti_star,Bx,By,mux,muy)

end subroutine compsums 

