C     Gateway subroutine
C
C     compsums_gtw.F
C
C     Invokes the Fortran COMPSUMS routine for WFRe
C      
C	  December 2011	
C
C   ---------------------------------------------------------
C
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

      integer plhs(*), prhs(*)
      integer mxGetPr, mxCreateDoubleMatrix, mxCreateNumericArray
      integer mxClassIDFromClassName
      integer nlhs, nrhs
      integer mxGetM, mxGetN
      integer s,t,x,y,rngs,rngt,nksv,nktv,orsv,ortv,Wmc0,Emc0,mx,my,Cx,
     .        Cy,s2x,s2y,A,Sw,sqSw,invSw,Se,sqSe,taux0,tauy0
      integer Sw0w0,Snormw2,Sr1_2,Sz0w0,SBBx,SBRx,Snormx2,SuuBBx,
     .        SuBxr6,SBBy,SBRy,Snormy2,SvvBBy,SvByr7,f,pred_w,pred_z
      integer nks,nkt,ors,ort,n,m1,m2,Nmc,dx,dy,px,py,qx,qy,rx,ry
      integer n_s,n_t,n_x,n_y,n_rngs,n_rngt,n_nksv,n_nktv,n_orsv,n_ortv,
     .        n_Wmc0,n_Emc0,n_mx,n_my,n_Cx,n_Cy,n_s2x,n_s2y,n_A,
     .        n_Sw,n_sqSw,n_invSw,n_Se,n_sqSe,n_taux0,n_tauy0
      integer m_s,m_t,m_x,m_y,m_rngs,m_rngt,m_nksv,m_nktv,m_orsv,m_ortv,
     .        m_Wmc0,m_Emc0,m_mx,m_my,m_Cx,m_Cy,m_s2x,m_s2y,m_A,m_Sw,
     .        m_sqSw,m_invSw,m_Se,m_sqSe,m_taux0,m_tauy0
      integer errflag, dims(3)

C     Check number of inputs and outputs
      if(nrhs .ne. 26) then
         call mexErrMsgTxt('26 inputs required.')
      elseif(nlhs .ne. 17) then
         call mexErrMsgTxt('17 outputs required.')
      endif

C     Load input pointers
	  s = mxGetPr(prhs(1))
	  t = mxGetPr(prhs(2)) 
	  x = mxGetPr(prhs(3))
	  y = mxGetPr(prhs(4))
	  rngs = mxGetPr(prhs(5))
	  rngt = mxGetPr(prhs(6))      
	  nksv = mxGetPr(prhs(7))
	  nktv = mxGetPr(prhs(8))
	  orsv = mxGetPr(prhs(9))
	  ortv = mxGetPr(prhs(10))
	  Wmc0 = mxGetPr(prhs(11))
	  Emc0 = mxGetPr(prhs(12))
	  mx = mxGetPr(prhs(13))
	  my = mxGetPr(prhs(14))
	  Cx = mxGetPr(prhs(15))
	  Cy = mxGetPr(prhs(16))
	  s2x = mxGetPr(prhs(17))
	  s2y = mxGetPr(prhs(18))
	  A = mxGetPr(prhs(19))
	  Sw = mxGetPr(prhs(20))
	  sqSw = mxGetPr(prhs(21))	  
	  invSw = mxGetPr(prhs(22))	  
	  Se = mxGetPr(prhs(23))	  
	  sqSe = mxGetPr(prhs(24))
	  taux0 = mxGetPr(prhs(25))
	  tauy0 = mxGetPr(prhs(26))

C     Check input dimensions        
      m_s = mxGetM(prhs(1))
      n_s = mxGetN(prhs(1))
      if((m_s.gt.1).and.(n_s.gt.1)) then
         call mexErrMsgTxt('Input S must be a vector.')
      endif
      m1 = max(m_s,n_s)
      m_t = mxGetM(prhs(2))
      n_t = mxGetN(prhs(2))
      if((m_t.gt.1).and.(n_t.gt.1)) then
         call mexErrMsgTxt('Input T must be a vector.')
      endif
      m2 = max(m_t,n_t)
      m_x = mxGetM(prhs(3))
      n_x = mxGetN(prhs(3))
      if(n_x.ne.m1) then
         call mexErrMsgTxt('#Columns of X must be length(S).')
      endif
      n = m_x
      m_y = mxGetM(prhs(4))
      n_y = mxGetN(prhs(4))
      if(m_y.ne.n) then
         call mexErrMsgTxt('#Rows(Y) must be equal to #rows(X).')
      endif
      if(n_y.ne.m2) then
         call mexErrMsgTxt('#Columns of Y must be length(T).')
      endif
      m_rngs = mxGetM(prhs(5))
      n_rngs = mxGetN(prhs(5))
      if((m_rngs.gt.1).and.(n_rngs.gt.1)) then
         call mexErrMsgTxt('Input RNGS must be a vector.')
      endif
      if(max(m_rngs,n_rngs).gt.2) then
         call mexErrMsgTxt('Length(RNGS) must be 2.')
      endif
      m_rngt = mxGetM(prhs(6))
      n_rngt = mxGetN(prhs(6))
      if((m_rngt.gt.1).and.(n_rngt.gt.1)) then
         call mexErrMsgTxt('Input RNGT must be a vector.')
      endif
      if(max(m_rngt,n_rngt).gt.2) then
         call mexErrMsgTxt('Length(RNGT) must be 2.')
      endif
      m_nksv = mxGetM(prhs(7))
      n_nksv = mxGetN(prhs(7))
      if((m_nksv.gt.1).and.(n_nksv.gt.1)) then
         call mexErrMsgTxt('Input NKSV must be a vector.')
      endif
      nks = max(m_nksv,n_nksv)
      m_nktv = mxGetM(prhs(8))
      n_nktv = mxGetN(prhs(8))
      if((m_nktv.gt.1).and.(n_nktv.gt.1)) then
         call mexErrMsgTxt('Input NKTV must be a vector.')
      endif
      nkt = max(m_nktv,n_nktv)
      m_orsv = mxGetM(prhs(9))
      n_orsv = mxGetN(prhs(9))
      if((m_orsv.gt.1).and.(n_orsv.gt.1)) then
         call mexErrMsgTxt('Input ORSV must be a vector.')
      endif
      ors = max(m_orsv,n_orsv)
      m_ortv = mxGetM(prhs(10))
      n_ortv = mxGetN(prhs(10))
      if((m_ortv.gt.1).and.(n_ortv.gt.1)) then
         call mexErrMsgTxt('Input ORTV must be a vector.')
      endif
      ort = max(m_ortv,n_ortv)
      dx = mxGetM(prhs(11))
      Nmc = mxGetN(prhs(11))
      dy = mxGetM(prhs(12))
      n_Emc0 = mxGetN(prhs(12))
      if((n_Emc0.ne.Nmc)) then
         call mexErrMsgTxt('#Cols(Emc0) must be = #cols(Wmc0).')
      endif
      m_mx = mxGetM(prhs(13))
      n_mx = mxGetN(prhs(13))
      if((m_mx.gt.1).and.(n_mx.gt.1)) then
         call mexErrMsgTxt('Input MX must be a vector.')
      endif
      qx = max(m_mx,n_mx)
      m_my = mxGetM(prhs(14))
      n_my = mxGetN(prhs(14))
      if((m_my.gt.1).and.(n_my.gt.1)) then
         call mexErrMsgTxt('Input MY must be a vector.')
      endif
      qy = max(m_my,n_my)
      m_Cx = mxGetM(prhs(15))
      px = mxGetN(prhs(15))
      if((m_Cx.ne.qx)) then
         call mexErrMsgTxt('#Rows(Cx) must be equal to length(MX).')
      endif
      m_Cy = mxGetM(prhs(16))
      py = mxGetN(prhs(16))
      if((m_Cy.ne.qy)) then
         call mexErrMsgTxt('#Rows(Cy) must be equal to length(MYs).')
      endif
      m_s2x = mxGetM(prhs(17))
      n_s2x = mxGetN(prhs(17))
      if((m_s2x.gt.1).or.(n_s2x.gt.1)) then
         call mexErrMsgTxt('Input S2X must be a scalar.')
      endif
      m_s2y = mxGetM(prhs(18))
      n_s2y = mxGetN(prhs(18))
      if((m_s2y.gt.1).or.(n_s2y.gt.1)) then
         call mexErrMsgTxt('Input S2Y must be a scalar.')
      endif
      m_A = mxGetM(prhs(19))
      n_A = mxGetN(prhs(19))
      if((m_A.ne.dy).or.(n_A.ne.dx)) then
         call mexErrMsgTxt('Wrong dimensions of A.')
      endif
      m_Sw = mxGetM(prhs(20))
      n_Sw = mxGetN(prhs(20))
      if((m_Sw.ne.dx).or.(n_Sw.ne.dx)) then
         call mexErrMsgTxt('Wrong dimensions of Sw.')
      endif
      m_sqSw = mxGetM(prhs(21))
      n_sqSw = mxGetN(prhs(21))
      if((m_sqSw.ne.dx).or.(n_sqSw.ne.dx)) then
         call mexErrMsgTxt('Wrong dimensions of sqSw.')
      endif
      m_invSw = mxGetM(prhs(22))
      n_invSw = mxGetN(prhs(22))
      if((m_invSw.ne.dx).or.(n_invSw.ne.dx)) then
         call mexErrMsgTxt('Wrong dimensions of invSw.')
      endif
      m_Se = mxGetM(prhs(23))
      n_Se = mxGetN(prhs(23))
      if((m_Se.ne.dy).or.(n_Se.ne.dy)) then
         call mexErrMsgTxt('Wrong dimensions of Se.')
      endif
      m_sqSe = mxGetM(prhs(24))
      n_sqSe = mxGetN(prhs(24))
      if((m_sqSe.ne.dy).or.(n_sqSe.ne.dy)) then
         call mexErrMsgTxt('Wrong dimensions of sqSe.')
      endif
      m_taux0 = mxGetM(prhs(25))
      n_taux0 = mxGetN(prhs(25))
      if((m_taux0.gt.1).and.(n_taux0.gt.1)) then
         call mexErrMsgTxt('Input TAUX0 must be a vector.')
      endif
      rx = max(m_taux0,n_taux0)
      m_tauy0 = mxGetM(prhs(26))
      n_tauy0 = mxGetN(prhs(26))
      if((m_tauy0.gt.1).and.(n_tauy0.gt.1)) then
         call mexErrMsgTxt('Input TAUY0 must be a vector.')
      endif
      ry = max(m_tauy0,n_tauy0)

      if(qx.ne.(nks+ors)) then
         call mexErrMsgTxt('Input QX, NKS and ORS inconsistent.')
      endif
      if(qy.ne.(nkt+ort)) then
         call mexErrMsgTxt('Input QT, NKT and ORT inconsistent.')
      endif

C     Create arrays for outputs
      plhs(1) = mxCreateDoubleMatrix(dx,dx,0)
      Sw0w0 = mxGetPr(plhs(1))
      plhs(2) = mxCreateDoubleMatrix(1,1,0)
      Snormw2 = mxGetPr(plhs(2))
      plhs(3) = mxCreateDoubleMatrix(dy,1,0)
      Sr1_2 = mxGetPr(plhs(3))
      plhs(4) = mxCreateDoubleMatrix(dy,dx,0)
      Sz0w0 = mxGetPr(plhs(4))
      plhs(5) = mxCreateDoubleMatrix(qx,qx,0)
      SBBx = mxGetPr(plhs(5))
      plhs(6) = mxCreateDoubleMatrix(qx,1,0)
      SBRx = mxGetPr(plhs(6))
      plhs(7) = mxCreateDoubleMatrix(1,1,0)
      Snormx2 = mxGetPr(plhs(7))
      dims(1) = qx
      dims(2) = qx
      dims(3) = px
      plhs(8) = mxCreateNumericArray(3,dims,
     .                      mxClassIDFromClassName("double"),0)
      SuuBBx = mxGetPr(plhs(8))
      plhs(9) = mxCreateDoubleMatrix(qx,px,0)
      SuBxr6 = mxGetPr(plhs(9))
      plhs(10) = mxCreateDoubleMatrix(qy,qy,0)
      SBBy = mxGetPr(plhs(10))
      plhs(11) = mxCreateDoubleMatrix(qy,1,0)
      SBRy = mxGetPr(plhs(11))
      plhs(12) = mxCreateDoubleMatrix(1,1,0)
      Snormy2 = mxGetPr(plhs(12))
      dims(1) = qy
      dims(2) = qy
      dims(3) = py
      plhs(13) = mxCreateNumericArray(3,dims,
     .                      mxClassIDFromClassName("double"),0)
      SvvBBy = mxGetPr(plhs(13))
      plhs(14) = mxCreateDoubleMatrix(qy,py,0)
      SvByr7 = mxGetPr(plhs(14))
      plhs(15) = mxCreateDoubleMatrix(n,1,0)
      f = mxGetPr(plhs(15))
      plhs(16) = mxCreateDoubleMatrix(n,dx,0)
      pred_w = mxGetPr(plhs(16))
      plhs(17) = mxCreateDoubleMatrix(n,dy,0)
      pred_z = mxGetPr(plhs(17))

C     Call the computational subroutine.
      call compsums(%val(s),%val(t),%val(x),%val(y),%val(rngs),
     . %val(rngt),nks,nkt,ors,ort,%val(Wmc0),%val(Emc0),
     . %val(mx),%val(my),%val(Cx),%val(Cy),%val(s2x),%val(s2y),
     . %val(A),%val(Sw),%val(sqSw),%val(invSw),%val(Se),%val(sqSe),
     . %val(taux0),%val(tauy0),n,m1,m2,Nmc,dx,dy,px,py,qx,qy,rx,ry,
     . %val(Sw0w0),%val(Snormw2),%val(Sr1_2),%val(Sz0w0),%val(SBBx),
     . %val(SBRx),%val(Snormx2),%val(SuuBBx),%val(SuBxr6),%val(SBBy),
     . %val(SBRy),%val(Snormy2),%val(SvvBBy),%val(SvByr7),%val(f),
     . %val(pred_w),%val(pred_z),errflag)

	  if (errflag.eq.1) then
	  	call mexErrMsgTxt('COMPSUMS is returning an error flag.')
	  end if
	  
      return
      end

