subroutine jupp2(v,a,b,F,h,errflag,m,n)
!Jupp transform / inverse Jupp transform.
!INPUT:
!   v   (m x n)     If F='direct', each row of v is a knot vector, non-decreasing,
!                     excluding interval endpoints (repeated knots allowed)                     
!                   If F='inverse', v are transformed knots
!   a,b (scalar)    Interval endpoints
!   F   (string)    If F='direct', Jupp transform is computed
!                   If F='inverse', inverse Jupp transform is computed
!
!OUTPUT:
!   h   (m x n)     Transformed knots / knots (depending on F)
!	errflag (int)	Error flag: 0 if there wer no errors, 1 otherwise
!
! Fortran version: December 2011

implicit none
! declare arguments
integer, intent(in) :: m, n
double precision, intent(in) :: v(m,n), a, b
 character(len=*), intent(in) :: F
double precision, intent(out) :: h(m,n)
integer, intent(out) :: errflag
! declare locals
integer :: i, j
double precision, allocatable :: vv(:), z(:), d(:,:)

errflag = 0

select case(F)	
    case ("direct")
    	allocate(vv(n+2))
    	vv(1) = a
    	vv(n+2) = b
    	do i = 1,m
    		vv(2:n+1) = v(i,:)
    		if (any(vv(1:n+1)>vv(2:n+2))) then
    			errflag = 1
    			print "(A)", " Knot sequence is not nondecreasing in JUPP"
   			end if
    		do j = 3,n+2
    			if (vv(j)>vv(j-1) .and. vv(j-1)>vv(j-2)) then
    				h(i,j-2) = log((vv(j)-vv(j-1))/(vv(j-1)-vv(j-2)))
    			else if (vv(j)>vv(j-1) .and. vv(j-1)==vv(j-2)) then
    				h(i,j-2) = huge(vv(j))
    			else if (vv(j)==vv(j-1) .and. vv(j-1)>vv(j-2)) then
    				h(i,j-2) = -huge(vv(j))
   				else
   					h(i,j-2) = 0.d0
				end if
			end do
		end do
		deallocate(vv)
    case ("inverse")
    	allocate(z(m),d(m,n))
        h = exp(v)
        z = 1.d0
        do i = 1,n
            z = 1.d0 + h(:,n+1-i)*z
        end do
        d = 0.d0
        d(:,1) = (b-a)/z
        do i = 1,n-1
            d(:,i+1) = d(:,i)*h(:,i)
        end do
        do i = 1,n
        	h(:,i) = a + sum(d(:,1:i),dim=2)
       	end do
       	deallocate(z,d)
    case default
        print "(A)", " Wrong input F in Jupp function"
        errflag = 1
end select

end subroutine jupp2

