function [ise,ypred] = cv5_WFRe(data,modpar,baspar,N,mxit,dsp)
% [ise,ypred] = cv5_WFRe(data,modpar,baspar,N,mxit,dsp)
% Five-fold cross-validation for Warped Functional Regression.
% Same input as WFRe.
% Output are individual cross-validated ISE's (n x 1) and predicted responses (n x nuy)
%
% External calls: WFRe, pred_WFRe
% Version: July 2014

[n,m] = size(data.y);
delta = [data.t(2)-data.t(1),(data.t(3:m)-data.t(1:m-2))/2,data.t(m)-data.t(m-1)];
ntest = ceil(n/5);
ise = NaN(n,1);
ypred = NaN(n,m);
for i = 1:5
    disp(['Training set ---> ' num2str(i)])
    itest = (i-1)*ntest+1:min(i*ntest,n);
    itrain = setdiff(1:n,itest);
    data_train = data;
    data_train.x = data.x(itrain,:);
    data_train.y = data.y(itrain,:);
    [modpar_train,f_tr,pred_tr,errflag_train] = WFRe(data_train,modpar,baspar,N,mxit,dsp);
    if ~errflag_train
        ypred(itest,:) = pred_WFRe(data.s,data.x(itest,:),data.t,modpar_train,baspar,N);
        ise(itest) = ((data.y(itest,:)-ypred(itest,:)).^2)*delta';
    else
        disp('DRFR produced an error flag for a train set')
        disp('No prediction possible for the corresponding test set')
    end
end
