function c = find_c(A,b,L)
%function c = find_c(A,b,L)
%Finds min c'Ac-2b'c subject to c'c=1, L'c=0 (L is optional argument)

%Input check
p = size(A,1);
if length(b)~=p
    disp('Dimensions of A and b not compatible in FIND_C')
    c = [];
    return
end
if size(b,2)>1
    b = b';
end

% Orthogonal reduction (if L is given)
if nargin==3
    if size(L,1)~=p
        disp('Dimensions of A and L not compatible in FIND_C')
        c = [];
        return
    end
    L1 = null(L');
    A = L1'*A*L1;
    b = L1'*b;
end

% Computation of minimizer
[U,D] = svd(A);
alpha = U'*b;
lmax = min(diag(D));
l = 0;
gl = sum(alpha.^2./(diag(D)-l).^2);
err = abs(gl-1);
iter = 0;
while err>1e-6 && iter<=50      % Find "l" that gives norm 1
    iter = iter+1;
    l0 = l;
    gl1 = sum(2*alpha.^2./(diag(D)-l).^3);
    l = min(l0+(1-gl)/gl1,lmax-1e-10);
    gl = sum(alpha.^2./(diag(D)-l).^2);
    err = abs(gl-1);
end
c = U*diag(1./(diag(D)-l))*alpha;
if nargin==3
    c = L1*c;
end
