function h = jupp(v,a,b,F)
%function h = jupp(v,a,b,F)
%Jupp transform / inverse Jupp transform.
%
%INPUT:
%   v   (m x n)     If F='direct', each row of v is a knot vector, non-decreasing,
%                     excluding interval endpoints (repeated knots allowed)                     
%                   If F='inverse', v are transformed knots
%   a,b (scalar)    Interval endpoints
%   F   (string)    If F='direct', Jupp transform is computed
%                   If F='inverse', inverse Jupp transform is computed
%
%OUTPUT:
%   h   (m x n)     Transformed knots / knots (depending on F)

[m,n] = size(v);
switch F
    case 'direct'
        d = [v,b*ones(m,1)] - [a*ones(m,1),v];
        h = log(d(:,2:n+1)./d(:,1:n));
        h(isnan(h)) = -Inf;
    case 'inverse'
        h = exp(v);
        z = ones(m,1);
        for i = 1:n
            z = ones(m,1) + h(:,n+1-i).*z;
        end
        d = zeros(m,n);
        d(:,1) = (b-a)./z;
        for i = 1:n-1
            d(:,i+1) = d(:,i).*h(:,i);
        end
        h = a + cumsum(d,2);
    otherwise
        error('Wrong input F')
end

